package electionSimulator;
import static org.junit.Assert.assertEquals;

import java.util.ArrayList;

import org.junit.Before;
import org.junit.Test;

/**
 * Tests the election simulator class.
 * 
 * @author Curt Clifton. Created Sep 23, 2008.
 */
public class ElectionSimulatorTest {

	private ElectionSimulator justIndiana;
	private ElectionSimulator neighbors;
	private ElectionSimulator blueland;
	private ElectionSimulator redland;

	/**
	 * Creates some scenarios for testing.
	 * 
	 * @throws java.lang.Exception
	 */
	@Before
	public void setUp() throws Exception {
		final State indiana = new State("Indiana", 11, .484, .497);
		final State illinois = new State("Illinois", 21, .582, .404);
		final State michigan = new State("Michigan", 17, .515, .461);
		final State ohio = new State("Ohio", 20, .491, .487);
		final State kentucky = new State("Kentucky", 8, .410, .578);

		ArrayList<State> states = new ArrayList<State>();
		states.add(indiana);
		this.justIndiana = new ElectionSimulator(states);

		states = new ArrayList<State>();
		states.add(indiana);
		states.add(illinois);
		states.add(michigan);
		states.add(ohio);
		states.add(kentucky);
		this.neighbors = new ElectionSimulator(states);

		states = new ArrayList<State>();
		states.add(new State("Blueland", 1, 1.0, 0.0));
		this.blueland = new ElectionSimulator(states);

		states = new ArrayList<State>();
		states.add(new State("Redland", 1, 0.0, 1.0));
		this.redland = new ElectionSimulator(states);
	}

	/**
	 * Test method for {@link ElectionSimulator#runSimulation()}.
	 */
	@Test
	public void testRunSimulation() {
		int votesForA = this.blueland.runSimulation();
		assertEquals(1, votesForA);

		votesForA = this.redland.runSimulation();
		assertEquals(0, votesForA);
	}

	/**
	 * Test method for {@link ElectionSimulator#runSimulations(int)}.
	 */
	@Test
	public void testRunSimulations() {
		double[] probabilities = this.justIndiana.runSimulations(10000);
		/*
		 * Just one state, so only 0 or its 11 electoral votes should have a
		 * non-zero probability.
		 */
		final double EPSILON = 0.01;
		assertEquals(0.4935, probabilities[11], EPSILON);
		assertEquals(0.5065, probabilities[0], EPSILON);
		for (int i = 1; i < 11; i++) {
			assertEquals(0.0, probabilities[i], EPSILON);
		}
	}

	/**
	 * Test method for {@link ElectionSimulator#getTotalElectoralVotes()}.
	 */
	@Test
	public void testGetTotalElectoralVotes() {
		assertEquals(11, this.justIndiana.getTotalElectoralVotes());
		assertEquals(77, this.neighbors.getTotalElectoralVotes());
	}

}
